@@ -5,7 +5,7 @@
 $::PROTOCOL = '(?:s?https?|ftp)';
 $::RXURL    = '(?:s?https?|ftp)://[-\\w.!~*\'();/?:@&=+$,%#]+' ;
 $::charset  = 'EUC-JP';
-$::version  = '1.1.5_0 ($Date: 2006/10/09 05:10:19 $)';
+$::version  = '1.1.6_1';
 %::form     = ();
 $::me       = $::postme = $ENV{SCRIPT_NAME};
 $::print    = ' 'x 10000; $::print = '';
@@ -133,32 +133,26 @@ sub init_globals{
 
     @::body_header = ( $::config{body_header}||'' );
 
-    @::menubar = (
-        &anchor( $::config{FrontPage} , undef  ) ,
-        &anchor( 'New'                , { a=>'new' } ) ,
+    %::menubar = (
+        '100_FrontPage' => &anchor($::config{FrontPage} , undef  ) ,
+        '200_New'       => &anchor('New'                , { a=>'new' } ) ,
+        '500_Tools'     => &anchor('Tools',{a=>'tools'},{ref=>'nofollow'}) ,
+        '600_Index'     => &anchor('Index',{a=>'recent'}) ,
     );
+    @::menubar = ();
 
     ### menubar ###
     unless( exists $::form{a} ){
-        &is_frozen() or
-            push(@::menubar,
-                &anchor('Edit',{
-                    a=>'edt',
-                    p=>( exists $::form{p} ? $::form{p} : $::config{FrontPage} )
-                },{
-                    rel=>'nofollow'
-                }) );
-        push( @::menubar ,
-            &anchor('Edit(Admin)',{
-                a=>'edt',
-                p=>( exists $::form{p} ? $::form{p} : $::config{FrontPage} ),
-                admin=>'admin'
-            },{rel=>'nofollow'}) );
+        my $curpage=exists $::form{p} ? $::form{p} : $::config{FrontPage};
+        unless( &is_frozen() ){
+            $::menubar{'300_Edit'} =
+                &anchor('Edit',{ a=>'edt', p=>$curpage},{rel=>'nofollow'});
+        }
+
+        $::menubar{'400_Edit(Admin)'} =
+            &anchor('Edit(Admin)',
+                { a=>'edt', p=>$curpage, admin=>'admin'},{rel=>'nofollow'});
     }
-    push(@::menubar ,
-        &anchor('Tools',{a=>'tools'},{ref=>'nofollow'}) ,
-        &anchor('Index',{a=>'recent'})
-    );
 
     @::copyright = (
         qq(Generated by <a href="http://wifky.sourceforge.jp">wifky</a> $::version with Perl $])
@@ -168,6 +162,8 @@ sub init_globals{
 
     %::preferences = (
         ' General Options' => [
+            { desc=>'script-revision '.$::version.' $Date: 2006/12/04 15:27:28 $' ,
+              type=>'rem' },
             { desc=>'The sitename', name=>'sitename', size=>40 },
             { desc=>'Enable link to file://...', name=>'locallink' ,
               type=>'checkbox' },
@@ -212,7 +208,7 @@ sub init_globals{
         '990_normal'     => \&block_normal ,
     );
     @::footer_plugin    = ( \&plugin_footnote_flush );
-    @::atodekaku_plugin = ( \&atodekaku_outline );
+    @::atodekaku_plugin = ( \&atodekaku_outline , \&unverb );
 
     @::outline = ();
 }
@@ -274,7 +270,6 @@ sub putenc{
 
 sub flush{
     grep( $_->( \$main::print ) , @::atodekaku_plugin );
-    $::print =~ s|\a((?:[0-9a-f][0-9a-f])*)\a|pack('h*',$1)|ges;
     print $::print;
 }
 
@@ -324,16 +319,19 @@ sub deyen{
     $s ;
 }
 
+sub mtimeraw{
+    my ($fn)=@_; $::mtime_cache{$fn} ||= (-f $fn ? ( stat($fn) )[9] : 0);
+}
+
 sub mtime{
-    my $fn=shift;
-    unless( exists $::mtime_cache{$fn} ){
-        my @stat=stat($fn) 
-            or return ($::mtime_cache{$fn}='0000/00/00 00:00:00');
-        my @tm=localtime($stat[9]);
-        $::mtime_cache{ $fn } = sprintf('%04d/%02d/%02d %02d:%02d:%02d'
-            , 1900+$tm[5],1+$tm[4],@tm[3,2,1,0]);
-    }
-    $::mtime_cache{$fn};
+    &ymdhms( &mtimeraw(@_) );
+}
+
+sub ymdhms{
+    my $tm=$_[0] or return '0000/00/00 00:00:00';
+    my @tm=localtime( $tm );
+    sprintf('%04d/%02d/%02d %02d:%02d:%02d'
+        , 1900+$tm[5],1+$tm[4],@tm[3,2,1,0])
 }
 
 sub cacheoff{
@@ -399,6 +397,9 @@ sub print_form{
         &puts('<input type="checkbox" name="to_freeze" value="1"');
         &is_frozen() and &puts('checked');
         &puts(' >freeze <input type="hidden" name="admin" value="admin">');
+        if( &mtimeraw($fname) ){
+            &puts('<input type="checkbox" name="sage" value="1">sage');
+        }
     }
     &puts('<input type="submit" name="a" value="Commit">');
 
@@ -408,7 +409,7 @@ sub print_form{
         &print_signarea('qassword');
     }
     &puts('<input type="submit" name="a" value="Upload"></p>');
-    
+
     if( my @attachments=&list_attachment( $::form{p} ) ){
         &puts('<p>');
         foreach my $attach (sort @attachments){
@@ -518,9 +519,8 @@ sub check_frozen{
     }
 }
 sub check_conflict{
-    my $stamp_time = &title2mtime($::form{p});
-    my $begin_time = $::form{stamp};
-    if( $begin_time ne $stamp_time ){
+    my $stamp_time = &mtimeraw(&title2fname($::form{p}));
+    if( $stamp_time > 0 && (my $begin_time = $::form{stamp}) ne $stamp_time ){
         die( "!Someone else modified this page on ${stamp_time}".
              " after you began to edit on ${begin_time}!"  );
     }
@@ -551,7 +551,7 @@ sub write_file{
     if( length( ref($body) ? $$body : $body ) <= 0 &&
         scalar( grep(index($_,"${fname}__")==0 , &directory())) == 0 )
     {
-        unlink $fname;
+        unlink($fname) or rmdir($fname);
         &cacheoff;
         0;
     }else{
@@ -586,7 +586,10 @@ sub save_config{
     while( my ($key,$val)=each %::config ){
         $val and push( @settings , '#'.$key."\t".&yen($val) );
     }
+    my $lock=&title2fname('..LOCK..');
+    mkdir($lock,0777) or die("!Disk full or file writing conflict (lockfile=$lock)!");
     &write_file( 'index.cgi' , join("\n", @settings) );
+    rmdir($lock);
 }
 
 sub action_query_delete{
@@ -614,19 +617,21 @@ sub action_query_delete{
 sub action_commit{
     my $lock=&title2fname($::form{p},'LOCK');
     if( ! mkdir($lock,0777) ){
-        &do_preview( &errmsg("!File writing conflict (lockfile=$lock)!") );
+        &do_preview( &errmsg("!Disk full or file writing conflict (lockfile=$lock)!") );
         return;
     }
     eval{
-        &check_frozen;
-        &check_conflict;
+        &check_frozen();
+        &check_conflict();
     };
     if( $@ ){
+        rmdir $lock;
         &do_preview( &errmsg($@) );
     }else{
-        &do_submit;
+        eval{ &do_submit(); };
+        rmdir $lock;
+        $@ and die($@);
     }
-    rmdir $lock;
 }
 
 sub action_preview{
@@ -692,8 +697,8 @@ sub action_tools{
                 foreach my $p (@{$i->{option}}){
                     &putenc('<input type="radio" name="%s" value="%s"%s>%s<br>'
                         , $i->{name}
-                        , $p->[0] 
-                        , ( defined($main::config{$i->{name}}) && 
+                        , $p->[0]
+                        , ( defined($main::config{$i->{name}}) &&
                             $main::config{$i->{name}} eq $p->[0]
                           ? ' checked' : '' )
                         , $p->[1] );
@@ -857,7 +862,8 @@ sub do_submit{
 
     defined($::hook_submit) and $::hook_submit->(\$title , \$::form{honbun});
     if( &write_object( $title , \$::form{honbun} ) ){
-        ( $::form{to_freeze} eq '1' ) and chmod 0444,$fn;
+        $::form{to_freeze} and chmod 0444,$fn;
+        $::form{sage} and utime($::form{stamp},$::form{stamp},$fn);
         &transfer_page;
     }else{
         &transfer_url($::me);
@@ -897,7 +903,8 @@ sub action_edit{
     my $title = $::form{p};
     &print_header(divclass=>'max',title=>'Edit');
     &begin_day("Edit: $title");
-    &print_form( $title , &enc(&read_object( $title )), &title2mtime($title) );
+    my $fn=&title2fname($title);
+    &print_form( $title , &enc(&read_file($fn)), &mtimeraw($fn) );
 
     if( &object_exists($::form{p}) && exists $::form{admin} ){
         &putenc('<h2>Rename</h2>
@@ -1022,6 +1029,16 @@ sub print_page{
 sub verb{
     "\a".unpack('h*',$_[0])."\a";
 }
+sub unverb{
+    ${$_[0]} =~ s|\a((?:[0-9a-f][0-9a-f])*)\a|pack('h*',$1)|ges;
+}
+sub strip_tag{
+    my $text=shift;
+    &unverb( \$text );
+    $text =~ s/\r?\n/ /g;
+    $text =~ s/\<[^\>]*\>//g;
+    $text;
+}
 
 sub verbatim{
     my $html=shift;
@@ -1033,8 +1050,13 @@ sub verbatim{
 }
 
 sub inner_link{
-    my ($symbol,$title,$sharp) = ($_[0] , split(/(?=#[pf][0-9mt])/,$_[1]) );
-    $title = &denc($title);
+    my ($session,$symbol,$title,$sharp)
+        = ($_[0], $_[1] , split(/(?=#[pf][0-9mt])/,$_[2]) );
+    if( $title =~ /^#/ ){
+        ($title,$sharp)=($session->{title},$title);
+    }else{
+        $title = &denc($title);
+    }
 
     if( &object_exists($title) ){
         &anchor( $symbol , { p=>$title } , undef , $sharp);
@@ -1047,13 +1069,16 @@ sub plugin_menubar{
     shift;
     $::menubar_printed=1;
     '<p class="adminmenu menubar">'. join("\r\n",
-        map('<span class="adminmenu">'.$_.'</span>',@_,@::menubar)).'</p>';
+        map('<span class="adminmenu">'.$_.'</span>',
+            @_, map( $::menubar{$_} , sort keys %::menubar) , @::menubar )
+    ).'</p>';
 }
 
 sub plugin_search{
     sprintf( '<div class="search_form"><form class="search" action="%s">
         <input class="search" type="text" name="keyword" size="20" value="%s">
-        <input class="search" type="submit" name="a" value="?">
+        <input type="hidden" name="a" value="?">
+        <input class="search" type="submit" value="?">
         </form></div>' ,
         $::me ,
         &enc(exists $::form{keyword} ? $::form{keyword} : '' ));
@@ -1064,14 +1089,10 @@ sub plugin_footnote{
     my $footnotetext=$session->{argv};
     push(@{$session->{footnotes}}, $footnotetext );
 
-    $footnotetext =~ s|\a((?:[0-9a-f][0-9a-f])*)\a|pack('h*',$1)|ges;
-    $footnotetext =~ s/\r?\n/ /g;
-    $footnotetext =~ s/\<[^\>]*\>//g;
-
     my $i=$#{$session->{footnotes}} + 1;
-    my %attr=( title=>$footnotetext );
+    my %attr=( title=>&strip_tag($footnotetext)  );
     $session->{index} and $attr{name}="fm${i}";
-    '<sup>' . 
+    '<sup>' .
     &anchor("*${i}", { p=> $::form{p} } , \%attr , "#ft${i}" ) .
     '</sup>' ;
 }
@@ -1086,7 +1107,7 @@ sub plugin_footnote_flush{
     foreach my $t (@{$footnotes}){
         ++$i;
         &puts('<p class="footnote">' ,
-            &anchor("*$i",{ p=>$::form{p} } , 
+            &anchor("*$i",{ p=>$::form{p} } ,
             ($session->{index} ? { name=>"ft$i"} : undef) ,
             "#fm$i"),
             "$t</p>");
@@ -1104,8 +1125,8 @@ sub atodekaku_outline{
     my $depth=-2;
     my $ss='';
     foreach my $p( @::outline ){
-        next if $p->{title} eq 'Header' || 
-                $p->{title} eq 'Footer' || 
+        next if $p->{title} eq 'Header' ||
+                $p->{title} eq 'Footer' ||
                 $p->{title} eq 'Sidebar' ;
 
         my $diff=$p->{depth} - $depth;
@@ -1116,14 +1137,14 @@ sub atodekaku_outline{
             $depth >= 0  and $ss .= "</li>\n" ;
             $ss .= '<li>';
         }
-        $ss .= &anchor( 
+        $ss .= &anchor(
             $p->{text} ,
             { p=>$p->{title} } ,
             undef ,
             $p->{sharp} );
         $depth=$p->{depth};
     }
-    $ss .= '</li></ul>' x ($depth-2);
+    $ss .= '</li></ul>' x ($depth+2);
 
     $$print =~ s/\x1B\(outline\)/$ss/g;
 }
@@ -1142,10 +1163,11 @@ sub ls_core{
         push(@list, map{
              +{ fname  => $_ ,
                 title  => &fname2title($_) ,
+                mtimeraw => &mtimeraw($_) ,
                 mtime  => &mtime($_)
               }
             }grep{
-                  exists $opt->{a} 
+                  exists $opt->{a}
                 ? ($_ =~ $pat )
                 : ($_ =~ $pat && !/^e2/ && -f $_ )
             }
@@ -1187,7 +1209,7 @@ sub ls{
     foreach my $p ( &ls_core(\%opt,@arg) ){
         $buf .= '<li>';
         exists $opt{l} and $buf .= '<tt>'.$p->{mtime}.' </tt>';
-        exists $opt{i} and $buf .= '<tt>['.scalar(@{$::dir_cache{ $p->{fname} }}).'] </tt>';
+        exists $opt{i} and $buf .= '<tt>'.scalar(@{$::dir_cache{ $p->{fname} }}).' </tt>';
 
         $buf .= &anchor( &enc($p->{title}) , { p=>$p->{title} } );
         $buf .= "</li>\r\n";
@@ -1277,12 +1299,14 @@ sub cr2br{
 }
 
 sub preprocess_innerlink1{ ### >>{ ... } ###
-    ${$_[0]} =~ s|&gt;&gt;\{([^\}]+)\}|&inner_link($1,$1)|ge;
+    my ($text,$session)=@_;
+    $$text =~ s|&gt;&gt;\{([^\}]+)\}|&inner_link($session,$1,$1)|ge;
 }
 
 sub preprocess_innerlink2{ ### [[ ... | ... ] ###
+    my ($text,$session)=@_;
     ${$_[0]} =~ s!\[\[(?:([^\|\]]+)\|)?(.+?)\]\]!
-        &inner_link(defined($1)?$1:$2,$2)!ge;
+        &inner_link($session,defined($1)?$1:$2,$2)!ge;
 }
 
 sub preprocess_outerlink1{ ### http://...{ ... } style ###
@@ -1338,6 +1362,11 @@ sub preprocess{
     $text;
 }
 
+sub headline{
+    my ($body,$n,$id)=@_;
+    &puts("<h$n" . ($id ? qq( id="$id") : '') . ">$body</h$n>" );
+}
+
 sub midashi{
     my ($depth,$text,$session)=(@_);
     $text = &preprocess($text,$session);
@@ -1357,7 +1386,7 @@ sub midashi{
         push( @main::outline ,
                 {
                   depth => $depth ,
-                  text  => $text ,
+                  text  => &strip_tag($text) ,
                   title => $session->{title} ,
                   sharp => "#p$tag"
                 }
@@ -1366,7 +1395,7 @@ sub midashi{
         $text =~ s/^\+/${tag}. /;
         $text = &anchor( '<span class="sanchor">' .
                          &enc($::config{"${cls}mark"}) .
-                         '</span>' 
+                         '</span>'
                   , { p     => $session->{title} }
                   , { class => "${cls}mark sanchor" }
                   , "#p${tag}"
@@ -1377,11 +1406,7 @@ sub midashi{
         }else{
             &puts(qq(<div class="x${cls}">));
         }
-        if( $session->{index} ){
-            &puts( qq(<h$h id="p$tag">$text</h$h>) );
-        }else{
-            &puts( qq(<h$h>$text</h$h>) );
-        }
+        &headline($text,$h,( $session->{index} ? "p$tag" : undef ) );
         if( $session->{main} ){
             &puts(qq(<div class="${cls}body x${cls}body">));
         }else{
